from inspect import getcallargs, getargspec

def ensure_params_under_instance_context(func):
    argspecs = getargspec(func)

    def wrapper(*args, **kwargs):
        combined_kwargs = {key: val for (key, val) in args[0].__dict__.items()
                           if key in argspecs.args}
        combined_kwargs.update(kwargs)
        getcallargs(func, *args, **combined_kwargs)
        return func(*args, **combined_kwargs)
    return wrapper

def ensure_params_under_instance_context_dict(*dict_names):
    def decorator(func):
        def wrapper(*args, **kwargs):
            combined_kwargs = {}
            for name in dict_names:
                combined_kwargs.update(getattr(args[0], name, {}))
            getcallargs(func, *args, **combined_kwargs)
            return func(*args, **combined_kwargs)
        return wrapper
    return decorator

if __name__ == '__main__':
    class Test(object):

        def __init__(self):
            self.a = 1
            self.b = 2
            self.context = {
                'a': 3,
                'b': 4,
            }

        @ensure_params_under_instance_context
        def f_1(self, a, b, c=1):
            print(a, b, c)

        @ensure_params_under_instance_context_dict('context')
        def f_2(self, a, b, c=1):
            print(a, b, c)

    test = Test()
    test.f_1()
    test.f_2()



import re
from http_request_randomizer.requests.proxy.requestProxy import RequestProxy
from http_request_randomizer.requests.errors.ProxyListException import ProxyListException
from .func_utils import ensure_params_under_instance_context

req_proxy = RequestProxy()

VARIABLE_CHECK = re.compile('{([A-Z0-9a-z])+}')
API_URL = 'https://api.iextrading.com'

class API(object):

    def __init__(self, api_name, *args):
        """
        route path is the route for the desired API endpoint
        for variables in path, simply wrap that string with '{}'

        Examples:
        ['1.0', 'stock', 'aapl', 'company'] =>
        https://api.iextrading.com/1.0/stock/aapl/company
        ['1.0', 'stock', '<symbol>', 'company'] =>
        https://api.iextrading.com/1.0/stock/{symbol}/company

        The variables will be required parameters for the call function.
        """
        self._base_path = [API_URL] + list(args)
        self._api_name = api_name

    def _call(self, path, *args, **kwargs):
        """
        path: a list path for subroute, variable format is the same as the base_path
        args, kwargs: arguments to fill in the variables in path
        """

        full_path = self._base_path + path
        variables = [segment[1:-1] for segment in full_path if VARIABLE_CHECK.match(segment)]
        params_dict = {variables[i]: arg for (i, arg) in enumerate(args)}
        remaining_args = variables[len(args):]
        for arg in remaining_args:
            if arg not in kwargs:
                raise ValueError('Missing argument {} for API {}'.format(arg, self._api_name))
            params_dict[arg] = kwargs[arg]

        url = "/".join(full_path)
        url = url.format(**params_dict)
        while True:
            try:
                response = req_proxy.generate_proxied_request(url)
                break
            except ProxyListException:
                req_proxy = RequestProxy()

        if response.status_code != 200:
            message = [response.text, 'API Call Error for url:', url]
            raise ValueError("\n".join(message))
        return response.json()

if __name__ == '__main__':
    stock_api = API('Stock', '1.0', 'stock', '{symbol}')
    result = stock_api._call(['company'], symbol='aapl')
    print(result)
    result = stock_api._call(['company'], symbol='aaapl')
    print(result)



import requests

from .utils import API
from .utils.func_utils import ensure_params_under_instance_context


class Stock(API):

    def __init__(self, symbol, **preferences):
        super(Stock, self).__init__('Stock', '1.0', 'stock')
        self._preferences = preferences
        self.symbol = symbol

    @classmethod
    def symbols(cls):
        url = 'https://api.iextrading.com/1.0/ref-data/symbols'
        return requests.get(url).json()

    def batch_request(self, types, symbols=None, range=None, **kwargs):
        raise NotImplementedError

    @ensure_params_under_instance_context
    def book(self, symbol):
        return self._call(['{symbol}', 'book'], symbol=symbol)

    @ensure_params_under_instance_context
    def chart(self, symbol, range=range):
        """
        Chart API
        does not support date range
        """
        return self._call(['{symbol}', 'chart', '{range}'],
                          symbol=symbol, range=range)

    @ensure_params_under_instance_context
    def company(self, symbol):
        """
        Company API
        """
        return self._call(['{symbol}', 'company'], symbol=symbol)

    @ensure_params_under_instance_context
    def delayed_quote(self, symbol):
        return self._call(['{symbol}', 'delayed-quote'], symbol=symbol)

    @ensure_params_under_instance_context
    def dividends(self, symbol, range):
        return self._call(['{symbol}', 'dividends', '{range}'],
                          symbol=symbol, range=range)

    @ensure_params_under_instance_context
    def earnings(self, symbol):
        return self._call(['{symbol}', 'earnings'], symbol=symbol)

    @ensure_params_under_instance_context
    def financials(self, symbol):
        return self._call(['{symbol}', 'financials'], symbol=symbol)

    @ensure_params_under_instance_context
    def stats(self, symbol):
        return self._call(['{symbol}', 'stats'], symbol=symbol)

    @ensure_params_under_instance_context
    def news(self, symbol, range=None):
        if range is None:
            return self._call(['{symbol}', 'news'], symbol=symbol)
        else:
            return self._call(['{symbol}', 'news', 'last', '{range}'],
                              symbol=symbol, range=range)

    @ensure_params_under_instance_context
    def peers(self, symbol):
        return self._call(['{symbol}', 'peers'], symbol=symbol)

    @ensure_params_under_instance_context
    def quote(self, symbol, **kwargs):
        return self._call(['{symbol}', 'quote'], symbol=symbol, **kwargs)

    @ensure_params_under_instance_context
    def splits(self, symbol, range=None):
        if range is None:
            return self._call(['{symbol}', 'splits'], symbol=symbol)
        else:
            return self._call(['{symbol}', 'splits', '{range}'],
                              symbol=symbol, range=range)


if __name__ == '__main__':
    print(Stock.symbols())
    apple = Stock('aapl')
    print(apple.book())
    print(apple.chart(range='5y'))
    print(apple.company())
    print(apple.delayed_quote())
    print(apple.dividends(range='5y'))
    print(apple.earnings())
    print(apple.financials())
    print(apple.stats())
    print(apple.news())
    print(apple.peers())
    print(apple.quote())
    print(apple.splits())



from os import path
from setuptools import setup, find_packages

# To use a consistent encoding
from codecs import open

# Get the long description from the README file
here = path.abspath(path.dirname(__file__))
with open(path.join(here, "README.rst"), encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="pyiex",

    version='0.0.7',

    author="Wentao Lu",
    author_email="wentao@wentaolu.com",

    description="Python wrapper for IEX financial API",
    long_description=long_description,

    url="https://github.com/wugui2020/pyIEX",

    license="LICENSE.txt",

    classifiers=[
        "Intended Audience :: Developers",
        "Topic :: Office/Business :: Financial",
        "Programming Language :: Python :: 2.7"
    ],

    keywords="python IEX",

    packages=find_packages(exclude=["docs", "tests"]),

    install_requires=[
        "http-request-randomizer==1.2.2",
    ],

    extras_require={
        "dev": ["check-manifest"],
        "test": ["coverage"],
    }
)
